"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenApiTypescriptPlugin = void 0;
const path = __importStar(require("path"));
const openapi_typescript_1 = __importDefault(require("openapi-typescript"));
const node_core_library_1 = require("@rushstack/node-core-library");
const chokidar = node_core_library_1.Import.lazy('chokidar', require);
const jsYaml = node_core_library_1.Import.lazy('js-yaml', require);
const PLUGIN_NAME = 'openapi-typescript-plugin';
class OpenApiTypescriptPlugin {
    constructor() {
        this.pluginName = PLUGIN_NAME;
    }
    apply(heftSession, heftConfiguration, options) {
        heftSession.hooks.build.tap(PLUGIN_NAME, (build) => {
            build.hooks.preCompile.tap(PLUGIN_NAME, (preCompile) => {
                preCompile.hooks.run.tapPromise(PLUGIN_NAME, async () => {
                    const jsonSchema = node_core_library_1.JsonSchema.fromFile(`${__dirname}/schemas/openapi-typescript-plugin.schema.json`);
                    jsonSchema.validateObject(options, 'config/heft.json');
                    await this._runOpenApiTypescriptAsync(options, heftSession, heftConfiguration, build.properties.watchMode);
                });
            });
        });
    }
    async _runOpenApiTypescriptAsync(options, heftSession, heftConfiguration, isWatchMode) {
        const resolvedEntries = [];
        for (const entry of options.entries) {
            resolvedEntries.push(Object.assign(Object.assign({}, entry), { resolvedSourcePath: path.resolve(heftConfiguration.buildFolder, entry.sourcePath), resolvedOutputPath: path.resolve(heftConfiguration.buildFolder, entry.outputPath) }));
        }
        const logger = heftSession.requestScopedLogger('openapi-typescript');
        await node_core_library_1.Async.forEachAsync(resolvedEntries, async (entry) => {
            await this._generateOpenApiTypescript(entry, logger);
        }, { concurrency: 5 });
        if (isWatchMode) {
            for (const entry of resolvedEntries) {
                const watcher = chokidar.watch(entry.resolvedSourcePath, { ignoreInitial: true });
                const boundGenerateOpenApiTypescriptFunction = this._generateOpenApiTypescript.bind(this, entry, logger);
                watcher.on('add', boundGenerateOpenApiTypescriptFunction);
                watcher.on('change', boundGenerateOpenApiTypescriptFunction);
                watcher.on('unlink', boundGenerateOpenApiTypescriptFunction);
                watcher.on('error', (error) => logger.emitError(error));
            }
        }
    }
    async _generateOpenApiTypescript(entry, logger) {
        let fileContents;
        try {
            fileContents = await node_core_library_1.FileSystem.readFileAsync(entry.resolvedSourcePath);
        }
        catch (error) {
            if (node_core_library_1.FileSystem.isNotExistError(error)) {
                logger.emitError(new Error(`OpenAPI file not found: ${entry.sourcePath}`));
                await node_core_library_1.FileSystem.deleteFileAsync(entry.resolvedOutputPath);
                return;
            }
            else {
                throw error;
            }
        }
        let parsedApiFile;
        try {
            if (entry.resolvedSourcePath.endsWith('.yaml')) {
                parsedApiFile = jsYaml.safeLoad(fileContents);
            }
            else if (entry.resolvedSourcePath.endsWith('.json')) {
                parsedApiFile = node_core_library_1.JsonFile.parseString(fileContents);
            }
            else {
                logger.emitError(new Error(`Unsupported OpenAPI file format: ${entry.sourcePath}`));
                return;
            }
        }
        catch (error) {
            logger.emitError(new Error(`Failed to parse OpenAPI file ${entry.sourcePath}: ${error}`));
            return;
        }
        let output;
        try {
            output = await (0, openapi_typescript_1.default)(parsedApiFile);
        }
        catch (error) {
            logger.emitError(new Error(`Error generating typescript from OpenAPI file ${entry.sourcePath}: ${error}`));
            return;
        }
        try {
            await node_core_library_1.FileSystem.writeFileAsync(entry.resolvedOutputPath, output, { ensureFolderExists: true });
        }
        catch (error) {
            logger.emitError(new Error(`Error writing typescript file to ${entry.outputPath}: ${error}`));
        }
    }
}
exports.OpenApiTypescriptPlugin = OpenApiTypescriptPlugin;
exports.default = new OpenApiTypescriptPlugin();
//# sourceMappingURL=index.js.map